/*
 Printer configurator for Linuxconf v1.6
 (1.0 released on 1.18r5)
 by Daniel Mealha Cabrita (dancab@conectiva.com)
 October 25th, 2000
*/

/*
 this module needs:
 rhs-printfilters-1.57-3cl (used by or part of RH's printtool)
 because the /usr/lib/rhs/rhs-printfilters/ files
*/

/*
 look at
 printer_common_data.*
 for defines' table
*/

/*
 module features:
 - edit/add/remove printers and their filter configuration
 - manages hosts allowed to access local queues
 - manages printers' queues
 - detects if lpd is being started automatically for current
   runlevel and, if it isn't and user confirms action,
   configures lpd for starting automatically
 - fills gui fields with common/default values so the user doesn't
   need to provide data (s)he doesn't know or gives hint on what
   kind of data he must provide
 - LPRng-aware (doesn't support newstyled printcap file, though)
*/

/*
 v1.0 first release in Linuxconf package (1.18r5).

 v1.1 released June 1st, 2000
 - GUI is changed. instead tall scrollable windows now we
   have 'selection by section'.
 - suggests an unique printer name instead always 'lp' as
   before.
 - data in printer properties' window is not lost after selecting
   a new filter (except the filter settings -- can't be avoided).
 - now the module can be directly called from the command line.
 - main menu is now recognized this way in treemenu mode.
   (the [+] now appears)
 - other minor fixes i don't remember

 v1.2 (1.21)
 - lprm program wasn't referred in printer.daemons file so user
   could not delete queue. fixed.
 - now if unable to create spool dir asks user if he wants to use
   the existing one (does not verify if it is really a directory,
   though).
 - fixed bug in the routine for executing external commands
   and returning their stdout (it could miss some characters).
 - now module is LPRng-aware (but doesn't support newstyled
   printcap file).
 - fixed the '-dBitsPerPixel' wrongly added in 'uniprint' drivers
   (should be appended only to other drivers)
 - changed types from FILE to FILE_CFG (oops.. it was the source
   of module compilation problems in Linuxconf 1.20)

 v1.3
 - _now_ printer module appears correctly in treemenu mode
   (so you can see how often i use treemenu mode)
 - no more hardcoded limit for total of printers the module can
   deal with (memory available is now the limit, so it's virtually
   unlimited).
 -> printfconf assimilated to printer module

 v1.4
 - file permitions for LPRng now seem to be 100% correct
 - memory invasion was occuring for large 'About: ' entries in
   printerdb file, corrected.
 - now module recognizes correctly ppa drivers in printerdb
   file (wrong printers were being reported even if the configfile
   was correct).
 - now fast_ascii is automatically disabled if a PPA filter is
   selected. -- the fast_ascii option also doesn't appear on
   configuration window (it would be bad if some 'smart' user
   re-enable it).

 v1.5  (1.21r6)
 - added printer test options (in printer properties window)
 - fixed usr filter button so now it appears in html mode
 - now rp (printcap entry) is always defined when remote queue
   (no longer ommited if it is 'lp')
 - now the module makes sure /var/spool/lpd exists

 v1.6 (present in 1.22)
 - now test jobs are sent correctly to printer with aliases.
 - for filters which had the same driver in some cases the configurator
   showed a different filter description. fixed. (better to say it was
   workarounded).
 - after selecting a new filter, in some cases, the configurator
   presented something like '-dBitsPerPixel=3' in color depth. fixed.
 - corrected queue manager, so status are shown correctly for LPRng too.

 v1.7 (1.23)
 - now user may specify a spool directory when creating a new printer,
   if (s)he don't wants the directory suggestion.
 - module now permits creating a printername with '|' (printer aliases),
   AND verification for duplicated name is smart now (ex.: it sees lp3
   as duplicated for blah|lp3)
 - now when adding a new remote printer, if LPRng is used, the tag
   lpd_bounce will be added, so the local filter may be used.
 - now printers' list no longer flashes (dialog is updated instead
   being completely rebuilt).

 v1.8
 - now the 'Network Authorizations' menu only appears if old lpr
   system is used (old_lpr-specific stuff, not used in lprng/etc).
 - fixed a missing free() somewhere.
 - now the if='' entry points to 'filter' when SMB and NCP printers,
   instead directly the smb/ncp filters (so the postscript may be
   translated into something else too). -- general.cfg file now is
   generated just after the printer is created (instead only after
   the filter is configured) this way smb/ncp printers will be
   accessible just after the printer entry is created (this is needed
   because the 'if=''' entry now points to 'filter' instead pointing
   directly to the smb/ncp filter.

 TODOs:
 - include management for lprng-style printcap files.


 */


/* ***********

	Check comments starting with // ### and fill appropriate code there

   ***********
*/
#pragma implementation
#include <stdio.h>
#include <string.h>
#include <translat.h>
#include "printer.h"
#include "printer.m"
#include <netconf.h>

MODULE_DEFINE_VERSION(printer);

PUBLIC MODULE_printer::MODULE_printer()
	: LINUXCONF_MODULE("printer")
{
	linuxconf_loadmsg ("printer",PACKAGE_REV);
}


static const char *keymenu=NULL;

PUBLIC void MODULE_printer::setmenu (
	DIALOG &dia,
	MENU_CONTEXT context)
{
	if (context == MENU_MISCSERV){
		keymenu = MSG_U(M_printer,"Printer");
		dia.new_menuitem ("printer","",keymenu);
	}
}

PUBLIC int MODULE_printer::domenu (
	MENU_CONTEXT context,
	const char *key)
{
	if (context == MENU_MISCSERV){
		if (key == keymenu){
                    printer_ed();
                    // ### Place the call to the edit function here
		}
	}
	return 0;
}


PUBLIC int MODULE_printer::dohtml (const char *key)
{
	int ret = LNCF_NOT_APPLICABLE;
	if (strcmp(key,"printer")==0){
		// ### Insert any menu and dialog here
		ret = 0;
	}
	return ret;
}


static void usage()
{
    xconf_error (MSG_U(T_USAGE
		,"linuxconf --modulemain printer usage\n"
		 "\n"
		 "    printer --option ...\n")
		);
}

PUBLIC void MODULE_printer::usage (SSTRINGS &tb)
{
	tb.add (new SSTRING (MSG_R(T_USAGE)));
}

PUBLIC int MODULE_printer::execmain (int argc , char *argv[], bool)
{
	int ret = LNCF_NOT_APPLICABLE;
	const char *pt = strrchr(argv[0],'/');
	if (pt != NULL){
		pt++;
	}else{
		pt = argv[0];
	}

        if (strcmp(pt,"printer")==0){
            ret = -1;
                if (argc == 1){
                    // ### Place call to main menu of the module
                    printer_ed();
                }else{
                    // ### Add some option parsing for the module
			::usage();
		}
	}
	return ret;
}


PUBLIC int MODULE_printer::probe (int level, int target, bool simul)
{
/*    printf("------\n");
    printf("level: %d\ntarget: %d\n", level, target);

    if(level==2){
        if(simul==true){
            net_prtlog(NETLOG_CMD, "i'm going to do nice tricks on your computer!\n");
            printf("simul==true\n");
        } else {
            net_title("fsdfaguoihrgior");
            net_prtlog(NETLOG_CMD, "i'm going to do nice tricks on your computer! .\n");
            printf("simul==false\n");
        }
    }
  */
    return(0);
}


static MODULE_printer printer;


// i deserve a rise
