/* this part manages the specific printer properties (opens props window, etc)
   deals with the config files inside the spool directory */

#include <stdlib.h>
#include <string.h>
#include "printer.h"
#include "printer.m"
#include "deez_parser_handle.h"
#include <sys/stat.h>

#include "printer_cfile.h"
#include "printer_smbncp-specific.h"

/* these are my_parms, first entry is the filename */
char ps_cfg_tag[]="postscript.cfg\0GSDEVICE=\0RESOLUTION=\0COLOR=\0PAPERSIZE=\0EXTRA_GS_OPTIONS=\0REVERSE_ORDER=\0PS_SEND_EOF=\0NUP=\0RTLFTMAR=\0TOPBOTMAR=\0\0";
char to_cfg_tag[]="textonly.cfg\0TEXTONLYOPTIONS=\0CRLFTRANS=\0TEXT_SEND_EOF=\0\0";
char ge_cfg_tag[]="general.cfg\0DESIRED_TO=\0PAPERSIZE=\0PRINTER_TYPE=\0ASCII_TO_PS=\0\0";

/* variables for editfilter field of editprinter window */
/* they're meant to be accessed only by
 editfilter_window(int which_printer, DIALOG &filterwindow)
 editprinter_window(int which_printer)
 (which are one window now)
 */
char f_printername[TYPICAL_STRING_SIZE];

/* GUI variables */
char f_filterdescription[BIG_STRING_SIZE];
SSTRING f_resolution;
SSTRING f_papersize;
SSTRING f_colordepth;
char f_sendeof;
char f_correctcrlf;
char f_fastascii;
SSTRING f_printedpages;
int f_horizmargins;
int f_vertmargins;
char f_extragsopt[TYPICAL_STRING_SIZE];

/*
     qual filtro,
     descrio do filtro,

     resoluo (300x300),
     papel(a4),
     prof.cores(default),
     enviar EOF apos job,
     corrigir efeito escada (CRLF),
     impresso veloz(sem filtro PS),
     pginas impressas/pg(1, 2, 4, 8),
     margens dir/esq (18),
     margens topo/baixo (18),
     opes extras p/o GS
     */


#define ps_gsdevice         ps_cfg[0].d
#define ps_resolution       ps_cfg[1].d
#define ps_color            ps_cfg[2].d
#define ps_papersize        ps_cfg[3].d
#define ps_extra_gs_options ps_cfg[4].d
#define ps_reverse_order    ps_cfg[5].d
#define ps_send_eof         ps_cfg[6].d
#define ps_nup              ps_cfg[7].d
#define ps_rtlftmar         ps_cfg[8].d
#define ps_topbotmar        ps_cfg[9].d

#define to_textonlyoptions  to_cfg[0].d
#define to_crlftrans        to_cfg[1].d
#define to_text_send_eof    to_cfg[2].d

#define ge_desired_to       ge_cfg[0].d
#define ge_papersize        ge_cfg[1].d
#define ge_printer_type     ge_cfg[2].d
#define ge_ascii_to_ps      ge_cfg[3].d

/* this is where_to_go */
struct t_rh_array{
    char d[TYPICAL_STRING_SIZE];
};

/* these properties don't store all printers simultaneously (unlike printcap ones),
   they only store for the currently-edited printer */
struct t_rh_array ps_cfg[10];
struct t_rh_array to_cfg[3];
struct t_rh_array ge_cfg[4];

/* protos */
void tellme_which_printer_it_is(char *given_gsdriver, char *given_colordepth, char *output_string);
char *return_filter_propertylist(char *which_printer, char *which_property);
void gimme_string_element(char *given_string, int which_element, char *output_string);
void tellme_fully_which_printer_it_is(char *given_gsdriver, char *given_colordepth, char *output_string, char *secondary_output);

/* writes 'printer.lp_something This is my filter description' to global linuxconf settings file */
/* this is necessary to know which printer was selected by user since now there are new entries
 in printerdb file which points to the same drivername, because this sometimes the
 wrong printer is showed after selection just because it uses the same GS driver */
void write_filter_description(int which_printer, const char *given_desc)
{
    char mtemp[TYPICAL_STRING_SIZE];
    char *zerothis;

    strcpy(mtemp, my_printcap[which_printer].printer_name);
    if((zerothis=strchr(mtemp, '|')))
        *zerothis=0;
    strcat(mtemp, "_filterdesc");
    linuxconf_replace ("printer", mtemp, given_desc);
    linuxconf_save();
}

/* reads filter description from global linuxconf settings file */
const char *read_filter_description(int which_printer)
{
    char mtemp[TYPICAL_STRING_SIZE];
    char *zerothis;

    strcpy(mtemp, my_printcap[which_printer].printer_name);
    if((zerothis=strchr(mtemp, '|')))
        *zerothis=0;
    strcat(mtemp, "_filterdesc");
    return(linuxconf_getval("printer", mtemp));
}

void remove_filter_description(int which_printer)
{
    char mtemp[TYPICAL_STRING_SIZE];
    char *zerothis;

    strcpy(mtemp, my_printcap[which_printer].printer_name);
    if((zerothis=strchr(mtemp, '|')))
        *zerothis=0;
    strcat(mtemp, "_filterdesc");
    linuxconf_removeall ("printer", mtemp);
    linuxconf_save();
}

/* creates a thing like:
 ##PRINTTOOL3## LOCAL uniprint NAxNA legal {} U_EpsonStylusColor stc_h {}
 from the currently selected printer (loaded to variables) */
void create_commented_printcap_config(char *output_string)
{
    char my_startentry[TYPICAL_STRING_SIZE];
    char my_color[TYPICAL_STRING_SIZE];
    char my_crlftrans[TYPICAL_STRING_SIZE];

    *output_string=0;

    if(*ps_color) {
        strcpy(my_color, ps_color);
    } else {
        strcpy(my_color, "Default");
    }
    if(*to_crlftrans){
        strcpy(my_crlftrans, to_crlftrans);
    } else {
        strcpy(my_crlftrans, "{}");
    }

    tellme_fully_which_printer_it_is(ps_gsdevice, ps_color, NULL, my_startentry);

    /* stair-stepping */
    sprintf(output_string, "##PRINTTOOL3## %s %s %s %s {} %s %s %s",
            ge_printer_type, ps_gsdevice, ps_resolution, ps_papersize,
            my_startentry, my_color, my_crlftrans);
}

/* sorts the given string list */
void sort_string_list(char *given_string_list)
{
    int my_size;
    char *read_point;
    char *my_buff;
    char my_last_string[TYPICAL_STRING_SIZE];
    char my_current_selection[TYPICAL_STRING_SIZE];
    char *write_point;

    *my_last_string=0;
    *my_current_selection=0;

    my_size=0;
    read_point=given_string_list;
    while(*read_point){
        while(*read_point++){
            my_size++;
        }
        my_size++;
    }
    my_size++;

    if((my_buff=(char *)malloc(my_size))){
        write_point=my_buff;

        while(*my_current_selection!=''){
            strcpy(my_current_selection, ""); /* certainly the last one */
            read_point=given_string_list;
            while(*read_point){
                if(strcmp(read_point, my_current_selection)<0){
                    if(strcmp(read_point, my_last_string)>0){
                        strcpy(my_current_selection, read_point);
                    }
                }
                read_point+=strlen(read_point)+1;
            }
            if(*my_current_selection!=''){
                strcpy(write_point, my_current_selection);
                write_point+=strlen(write_point)+1;
                strcpy(my_last_string, my_current_selection);
            }
        }
        *write_point=0;
    }

    memcpy(given_string_list, my_buff, my_size);
    free(my_buff);
}

/* load work variables to gui variables
 (because the char-SSTRING conversion, the char-string conversion, etc) */
/* (remember when creating writting routine: mirror values between archives */
void load_to_gui_variables(void)
{
    /* loads filter description */
    {
        char *temps;

        temps=return_filter_propertylist(f_printername, "About: ");
        gimme_string_element(temps, 0, f_filterdescription);
        free(temps);
    }

    f_resolution.setfrom(ps_resolution);
    f_papersize.setfrom(ps_papersize);
    f_colordepth.setfrom(ps_color);
    f_printedpages.setfrom(ps_nup);

    if(!strcmp(ps_send_eof, "NO")){
        f_sendeof=0;
    } else {
        f_sendeof=1;
    }

    if(!strcmp(to_crlftrans, "1")){
        f_correctcrlf=1;
    } else {
        f_correctcrlf=0;
    }

    if(!strcmp(ge_ascii_to_ps, "NO")){
        f_fastascii=1;
    } else {
        f_fastascii=0;
    }

    f_horizmargins=atoi(ps_rtlftmar);
    f_vertmargins=atoi(ps_topbotmar);

    /* copy but eliminates first and last characters (those ") */
    strcpy(f_extragsopt, ps_extra_gs_options+1);
    {
        int tempi;

        if((tempi=strlen(f_extragsopt))){
            *(f_extragsopt+tempi-1)=0;
        }
    }
}

/* same as load_to_gui_variables, but in an opposite way :) */
void save_from_gui_variables(void)
{
    strcpy(ps_resolution, f_resolution.get());
    strcpy(ps_papersize, f_papersize.get());
    strcpy(ps_color, f_colordepth.get());
    strcpy(ps_nup, f_printedpages.get());

    if(f_sendeof){
        *ps_send_eof=0;
    } else {
        strcpy(ps_send_eof, "NO");
    }

    if(f_correctcrlf){
        strcpy(to_crlftrans, "1");
    } else {
        *to_crlftrans=0;
    }

    if(f_fastascii){
        strcpy(ge_ascii_to_ps, "NO");
    } else {
        *ge_ascii_to_ps=0;
    }

    sprintf(ps_rtlftmar, "%d", f_horizmargins);
    sprintf(ps_topbotmar, "%d", f_vertmargins);

    /* puts the " back as needed for config file */
    sprintf(ps_extra_gs_options, "\"%s\"", f_extragsopt);
}

/* given a string pointer, seeks for 0 and, then, returns the 0's
   address+1 (beginning of next string) */
/* returns NULL if that address also contains zero (end of string series) */
char *gimme_next_string(char *from_this_point)
{
    char *my_counter;

    my_counter=from_this_point;
    while(*(my_counter++));
    if(!(*my_counter))
        my_counter=NULL;

    return(my_counter);
}

/* which_printer is the string just after 'Description: {' */
char *return_filter_propertylist(char *which_printer, char *which_property)
{
    FILE_CFG *my_file;
    int  myfilesize;
    char *my_buff;
    char *my_out_buff=NULL;
    char *my_secondary_buff;
    char *where_to_write;
    char the_printername[500];
    char *read_from_here;

//    if((my_file=fopen(PRINTERDB_FILENAME, "r"))){
    if((my_file=cf_printerdbfile.fopen("r"))){
        fseek(my_file, 0, SEEK_END);
        myfilesize=ftell(my_file);
        fseek(my_file, 0, SEEK_SET);
        if((my_buff=(char *)malloc(myfilesize+1))){
            *my_buff=0;
            *(my_buff+myfilesize)=0;
            fread(my_buff, myfilesize, 1, my_file);
            if((my_out_buff=(char *)malloc(myfilesize+1))){
                where_to_write=my_out_buff;
                *where_to_write=0;
                if((my_secondary_buff=(char *)malloc(myfilesize+1))){
                    int i;

                    i=0;
                    while(copy_specific_block(my_buff, "StartEntry: ", i++, my_secondary_buff)){
                        parse_string_data(my_secondary_buff, "Description: {", "}", the_printername);
                        if(!(strcmp(the_printername, which_printer))){
                            read_from_here=my_secondary_buff;
                            while((read_from_here=parse_string_data(read_from_here, which_property, "\n", where_to_write)))
                            {
                                where_to_write+=strlen(where_to_write)+1;
                            }
                            *where_to_write=0; /* makes the 2nd zero, meaning the list finished */
                        }
                    }
                }
            }
            free(my_buff);
        }
//        fclose(my_file);
        cf_printerdbfile.fclose(my_file);
    }
    return(my_out_buff);
}

/* allocate a string in memory and return its pointer
 containing all the printer filters' names.
 name1\0name2\0ame3\0.. ..\0last_name\0\0  */
/* it MUST be freed after then! free(blabla) */
char *return_filters_list(void)
{
    FILE_CFG *my_file;
    int  myfilesize;
    char *my_buff;
    char *my_out_buff=NULL;
    char *my_secondary_buff;
    char *where_to_write;

//    if((my_file=fopen(printerdb_filename, "r"))){
    if((my_file=cf_printerdbfile.fopen("r"))){
        fseek(my_file, 0, SEEK_END);
        myfilesize=ftell(my_file);
        fseek(my_file, 0, SEEK_SET);
        if((my_buff=(char *)malloc(myfilesize+1))){
            *my_buff=0;
            *(my_buff+myfilesize)=0;
            fread(my_buff, myfilesize, 1, my_file);
            if((my_out_buff=(char *)malloc(myfilesize+1))){
                where_to_write=my_out_buff;
                *where_to_write=0;
                if((my_secondary_buff=(char *)malloc(myfilesize+1))){
                    int i;

                    i=0;
                    while(copy_specific_block(my_buff, "StartEntry: ", i++, my_secondary_buff)){
                        parse_string_data(my_secondary_buff, "Description: {", "}", where_to_write);
                        where_to_write+=strlen(where_to_write)+1;
                    }
                    *where_to_write=0; /* makes the 2nd zero, meaning the list finished */
                }
            }
            free(my_buff);
        }
//        fclose(my_file);
        cf_printerdbfile.fclose(my_file);
    }

    /* sorts filterlist.. */
    sort_string_list(my_out_buff);

    return(my_out_buff);
}

/* if add_export=true, append a 'export ' before each entry */
void save_rhprt_config(char *my_parms, struct t_rh_array *where_to_go, char *printer_spool_path, int add_export)
{
    FILE *my_file;
    char whole_filename[500];
    char *my_pointer;
    int  my_counter;
    char data_to_file[500];

    my_pointer=my_parms;
    my_counter=0;

    sprintf(whole_filename, "%s/%s", printer_spool_path, my_parms);

    if((my_file=fopen(whole_filename, "w"))){
        fputs("#\n# This file was generated by Linuxconf's Printer module\n# according fileformat from RH's printtool\n#\n# Comments and unsupported entries may be lost!\n#\n\n", my_file);
        while((my_pointer=gimme_next_string(my_pointer))){
            if(add_export){
                sprintf(data_to_file, "export %s%s\n", my_pointer, where_to_go[my_counter++].d);
            } else {
                sprintf(data_to_file, "%s%s\n", my_pointer, where_to_go[my_counter++].d);
            }
            fputs(data_to_file, my_file);
        }
        fclose(my_file);
        /* if the lprng crap is installed.. */
        if(kind_of_lpr_system_installed==LPRTYPE_LPRNG){
            change_owner_and_group(whole_filename, "daemon", "daemon");
            chmod(whole_filename, 384); /* -rw------- */
        }else{
            chmod(whole_filename, 493); /* -rwxr-xr-x */
        }
    }
}

/* load RH's printer config file to previously-allocated memory */
/* returns 0 if ok */
int load_rhprt_config(char *my_parms, struct t_rh_array *where_to_go, char *printer_spool_path)
{
    FILE *my_file;
    int  myfilesize;
    char *my_buff;
    char *my_pointer;
    int  my_counter;
    char whole_filename[500];

    my_pointer=my_parms;
    my_counter=0;

    sprintf(whole_filename, "%s/%s", printer_spool_path, my_parms);

    if((my_file=fopen(whole_filename, "r"))){
        fseek(my_file, 0, SEEK_END);
        myfilesize=ftell(my_file);
        fseek(my_file, 0, SEEK_SET);
        if((my_buff=(char *)malloc(myfilesize+1))){
            *my_buff=0;
            *(my_buff+myfilesize)=0;
            fread(my_buff, myfilesize, 1, my_file);

            while((my_pointer=gimme_next_string(my_pointer))){
                parse_string_data(my_buff, my_pointer, "\n", where_to_go[my_counter++].d);
            }
            free(my_buff);
        }
        fclose(my_file);
        return(0);
    }
    return(1);
}

/* load config files in printer spool directory to memory
   if they don't exist, create them with defaults */
void load_initialise_secondary_configfiles(int which_printer)
{
    char tempy[500];

    /* creates filterscript in spool directory, if absent */
    sprintf(tempy, "%s/%s", my_printcap[which_printer].sd, FILTERSCRIPT_NAME);
    if(!this_file_exists(tempy)){
        FILE *my_filter;
        FILE_CFG *my_master_filter;
        int  myfilesize;
        char *my_buff;

        if((my_filter=fopen(tempy, "w"))){
            if((my_master_filter=cf_masterfilterfile.fopen("r"))){
            // if((my_master_filter=fopen(MASTERFILTER_FULLPATH, "r"))){
                /* read file to memory.. */
                fseek(my_master_filter, 0, SEEK_END);
                myfilesize=ftell(my_master_filter);
                fseek(my_master_filter, 0, SEEK_SET);

                if((my_buff=(char *)malloc(myfilesize+1))){
                    if(fread(my_buff, myfilesize, 1, my_master_filter)){
                        *(my_buff+myfilesize)=0;
                        /* write it.. */
                        fputs(my_buff, my_filter);
                    }
                    cf_masterfilterfile.fclose(my_master_filter);
                    free(my_buff);
                }else{
                    cf_masterfilterfile.fclose(my_master_filter);
                }

            } else { /* this is unlikely to happen, but it will put my own filter instead if RH's one is not accessible */
                fputs("#!/bin/bash\n\n# this filterscript was generated by Linuxconf's Printer module\n", my_filter);
                fputs("# currently it only uses the postscript.cfg file, but it may change in future\n\n", my_filter);
                fputs("export SPOOLDIR=$(pwd)\n\nsource ${SPOOLDIR}/postscript.cfg\n# source ${SPOOLDIR}/general.cfg\n# source ${SPOOLDIR}/textonly.cfg\n", my_filter);
                fputs("\n", my_filter);
                fputs("if [ \"$GSDEVICE\" != \"TEXT\" ]; then\n", my_filter);
                fputs("   if [ \"$RESOLUTION\" != \"NAxNA\" ]; then\n", my_filter);
                fputs("      nenscript -TA4 -ZB -p- |\n", my_filter);
                fputs("      gs -q -sDEVICE=$GSDEVICE \\\n", my_filter);
                fputs("              -r$RESOLUTION \\\n", my_filter);
                fputs("              -sPAPERSIZE=$PAPERSIZE \\\n", my_filter);
                fputs("              -dNOPAUSE \\\n              -dSAFER \\\n", my_filter);
                fputs("              $COLOR \\\n", my_filter);
                fputs("              $EXTRA_GS_OPTIONS \\\n", my_filter);
                fputs("              -sOutputFile=- -\n", my_filter);
                fputs("   else\n", my_filter);
                fputs("      nenscript -TA4 -ZB -p- |\n", my_filter);
                fputs("      gs -q -sDEVICE=$GSDEVICE \\\n", my_filter);
                fputs("              -sPAPERSIZE=$PAPERSIZE \\\n", my_filter);
                fputs("              -dNOPAUSE \\\n              -dSAFER \\\n", my_filter);
                fputs("              $COLOR \\\n", my_filter);
                fputs("              $EXTRA_GS_OPTIONS \\\n", my_filter);
                fputs("              -sOutputFile=- -\n", my_filter);
                fputs("   fi\n", my_filter);
                fputs("else\n   cat -\nfi\n\n", my_filter);

                fputs("\nif [ \"$PS_SEND_EOF\" == \"\" ]; then\n", my_filter);
                fputs("        printf \"\\004\"\nfi\n", my_filter);
            }
            fclose(my_filter);

            if(kind_of_lpr_system_installed==LPRTYPE_LPRNG){
//                chmod(tempy, 384); /* -rw------- */
                chmod(tempy, 493); /* -rwxr-xr-x */
                change_owner_and_group(tempy, "daemon", "daemon");
            }else{
                chmod(tempy, 493); /* -rwxr-xr-x */
            }
        }
    }

    /* postscript.cfg */
    sprintf(tempy, "%s/%s", my_printcap[which_printer].sd, ps_cfg_tag);
    if(!this_file_exists(tempy)){
        strcpy(ps_gsdevice, "TEXT");
        strcpy(ps_resolution, "NAxNA");
        *ps_color=0;
        strcpy(ps_papersize, "a4");
        strcpy(ps_extra_gs_options, "\"\"");
        *ps_reverse_order=0;
        strcpy(ps_send_eof, "NO");
        strcpy(ps_nup, "1");
        strcpy(ps_rtlftmar, "18");
        strcpy(ps_topbotmar, "18");
    } else {
        load_rhprt_config(ps_cfg_tag, ps_cfg, my_printcap[which_printer].sd);
        /* adjust ps_color data */
        {
            char *where_it_should_begin;
            char tempy2[TYPICAL_STRING_SIZE];

            if(*ps_color){
                if((where_it_should_begin=strchr(ps_color, '='))){
                    strcpy(tempy2, ++where_it_should_begin);
                    strcpy(ps_color, tempy2);
                }
            }
        }
    }

    /* textonly.cfg */
    sprintf(tempy, "%s/%s", my_printcap[which_printer].sd, to_cfg_tag);
    if(!this_file_exists(tempy)){
        *to_textonlyoptions=0;
        *to_crlftrans=0;
        strcpy(to_text_send_eof, "NO");
    } else {
        load_rhprt_config(to_cfg_tag, to_cfg, my_printcap[which_printer].sd);
    }

    /* general.cfg */
    sprintf(tempy, "%s/%s", my_printcap[which_printer].sd, ge_cfg_tag);
    if(!this_file_exists(tempy)){
        strcpy(ge_desired_to, "ps");
        strcpy(ge_papersize, "a4");
        // strcpy(ge_printer_type, "LOCAL"); /* not defined here */
        strcpy(ge_ascii_to_ps, "NO");
    } else {
        load_rhprt_config(ge_cfg_tag, ge_cfg, my_printcap[which_printer].sd);
    }

    /* define everything defined before (from printcap) */
    strcpy(ge_printer_type, "LOCAL");
    switch(return_printer_type(which_printer)){
    case PRT_REMOTE:
        strcpy(ge_printer_type, "REMOTE");
        break;
    case PRT_SMBWIN:
        strcpy(ge_printer_type, "SMB");
        break;
    case PRT_NETWARE:
        strcpy(ge_printer_type, "NCP");
        break;
    }

    /* patches the -dBitsPerPixel=NONONONO thing */
    {
        char tempz[TYPICAL_STRING_SIZE];
        char *where_copy_from;

        strcpy(tempz, ps_color);
        if(*ps_color){
            if((where_copy_from=strchr(ps_color, '=')))
                strcpy(tempz, ++where_copy_from);
        }

        /* if the filter description is not found in internal linuxconf configfile..
           try to guess based on the filtername */
        {
            const char *readen_filterdesc;

            /* ok, now let's get the filter description from linuxconf global configfile
               _or_ guess it */
            if((readen_filterdesc=read_filter_description(which_printer))){
                strcpy(f_printername, readen_filterdesc);
            }else{
                tellme_which_printer_it_is(ps_gsdevice, tempz, f_printername);
            }
        }
    }
}

/* dump all the current printer's filter data to right files.. */
/* this what is called when user clicked the button for saving preferences */
void save_all_the_filters_stuff(int which_printer)
{
    save_from_gui_variables();

    /* adjust ps_color data, attach '-dBitsPerPixel=%s'
       but NOT if postscript driver is 'uniprint' */
    if((*ps_color)&&(strcmp(ps_gsdevice, "uniprint")&&(strcmp(ps_gsdevice, "ppa")))){
        char tempy2[TYPICAL_STRING_SIZE];

        sprintf(tempy2, "-dBitsPerPixel=%s", ps_color);
        strcpy(ps_color, tempy2);
    }
    /* postscript.cfg */
    save_rhprt_config(ps_cfg_tag, ps_cfg, my_printcap[which_printer].sd, 0);

    /* textonly.cfg */
    save_rhprt_config(to_cfg_tag, to_cfg, my_printcap[which_printer].sd, 0);

    /* general.cfg */
    save_rhprt_config(ge_cfg_tag, ge_cfg, my_printcap[which_printer].sd, 1);

    /* remove the heading '-dBitsPerPixel=' from ps_color, if it was previously added */
    if(*ps_color){
        char tempz[TYPICAL_STRING_SIZE];
        char *where_copy_from;

        strcpy(tempz, ps_color);
        if((where_copy_from=strchr(ps_color, '='))){
            strcpy(tempz, ++where_copy_from);
            strcpy(ps_color, tempz);
        }
    }
}

/* fills with right printername (Description:) from given specs */
void tellme_which_printer_it_is(char *given_gsdriver, char *given_colordepth, char *output_string)
{
    tellme_fully_which_printer_it_is(given_gsdriver, given_colordepth, output_string, NULL);
}

/* fills with right printername (Description:) from given specs */
/* if secondary output not null, it also returns the StartEntry: field */
void tellme_fully_which_printer_it_is(char *given_gsdriver, char *given_colordepth, char *output_string, char *secondary_output)
{
    FILE_CFG *my_file;
    int  myfilesize;
    char *my_buff;
    char *my_secondary_buff;
    char seek_for[TYPICAL_STRING_SIZE];
    char string_ends_with[10];
    char matchs_with[TYPICAL_STRING_SIZE];
    char temp_output[500];
    char *read_from_here;

    if(output_string)
        *output_string=0;
    if(secondary_output)
        *secondary_output=0;

    /* if not 'ppa' nor 'uniprint', normal filter */
    if((strcmp(given_gsdriver, "uniprint"))&&(strcmp(given_gsdriver, "ppa"))){
        strcpy(seek_for, "GSDriver: ");
        sprintf(string_ends_with, "\n");
        strcpy(matchs_with, given_gsdriver);
    }else{
        strcpy(seek_for, "BitsPerPixel: {");
        strcpy(string_ends_with, "}");
        strcpy(matchs_with, given_colordepth);
    }

//    if((my_file=fopen(printerdb_filename, "r"))){
    if((my_file=cf_printerdbfile.fopen("r"))){
        fseek(my_file, 0, SEEK_END);
        myfilesize=ftell(my_file);
        fseek(my_file, 0, SEEK_SET);
        if((my_buff=(char *)malloc(myfilesize+1))){
            *my_buff=0;
            *(my_buff+myfilesize)=0;
            fread(my_buff, myfilesize, 1, my_file);
            if((my_secondary_buff=(char *)malloc(myfilesize+1))){
                int i;

                i=0;
                while(copy_specific_block(my_buff, "StartEntry: ", i++, my_secondary_buff)){
                    read_from_here=my_secondary_buff;
                    while((read_from_here=parse_string_data(read_from_here, seek_for, string_ends_with, temp_output))){
                        if(!strcmp(matchs_with, temp_output)){
                            if(output_string)
                                parse_string_data(my_secondary_buff, "Description: {", "}", output_string);
                            if(secondary_output)
                                parse_string_data(my_secondary_buff, "StartEntry: ", "\n", secondary_output);
                        }
                    }

                }
            }
            free(my_buff);
        }
//        fclose(my_file);
        cf_printerdbfile.fclose(my_file);
    }
}

/* given a string
 {blablabla0} {blablabla1} {blablabla2} ..
 copies the specified element between the {}
 (element1 means 'blablabla1')
 */
void gimme_string_element(char *given_string, int which_element, char *output_string)
{
    char *my_pointer;
    char *my_output_pointer;
    int  my_counter;

    my_pointer=given_string-1;
    my_counter=which_element+1;
    my_output_pointer=output_string;

    while((my_counter--)&&my_pointer){
        my_pointer=strchr(++my_pointer, '{');
    }
    if((my_pointer)){
        my_pointer++;
        while(my_pointer&&(*my_pointer)&&(*my_pointer!='}')){
            *my_output_pointer++=*my_pointer++;
        }
    }
    *my_output_pointer=0;
}

/* filed popup menus with printerdb data, according the printer selected */
/* processing:
 0-none
 1-{300} {300} {}            to  300x300
 2-{blabla0} {blablabla..}   to  blabla0, blablabla..
 */
void populate_fieldlist(char *printer_id, char *field_identificator, FIELD_LIST *field_thing, int processing)
{
    char *generated_list;
    char *readpoint;
    char my_desc[500];
    char my_retcod[500];

    generated_list=return_filter_propertylist(printer_id, field_identificator);
    readpoint=generated_list;
    if(!*readpoint)
        field_thing->addopt("", MSG_U(M_NOTAVAIL, "(not available)"), "");

    while(*readpoint){
        switch(processing){
        case 0:
            strcpy(my_desc, readpoint);
            strcpy(my_retcod, readpoint);
            break;
        case 1:
            {
                char xres[20],
                     yres[20],
                     commentres[100];

                gimme_string_element(readpoint, 0, xres);
                gimme_string_element(readpoint, 1, yres);
                gimme_string_element(readpoint, 2, commentres);
                sprintf(my_retcod, "%sx%s", xres, yres);
                if(*commentres){
                    sprintf(my_desc, "%s (%s)", my_retcod, commentres);
                } else {
                    sprintf(my_desc, "%s", my_retcod);
                }
            }
            break;
        case 2:
            {
                gimme_string_element(readpoint, 0, my_retcod);
                gimme_string_element(readpoint, 1, my_desc);
            }
            break;

        }
        if(!strcmp("NAxNA", my_desc))
            strcpy(my_desc, MSG_U(M_NOTAVAIL2, "(not available)"));
        field_thing->addopt(my_retcod, my_desc, "");
        readpoint+=strlen(readpoint)+1;
    }
    free(generated_list);
}

/* opens a window listing all the printer filters available
 if filter selected, returns true and writes the filtername
 if filter wasn't selected, returns false */
/* the name returned is the printerdb 'Description' field */
int select_printer_filter(char *output_printer_filter)
{
    DIALOG_RECORDS dia_filter_list;
    MENU_STATUS my_button;
    int my_selection;
    char *whole_list;
    char *steppy;
    int  my_counter;

my_button=(MENU_STATUS)0;  /* just for killing those warnings.. */

    if((whole_list=return_filters_list())){
        dia_filter_list.newf_head("", MSG_U(M_FILTERNAME, "filter name"));

        steppy=whole_list;
        while(*steppy){
            dia_filter_list.new_menuitem(steppy, "");
            steppy+=strlen(steppy)+1;
        }
        my_selection=0;
        my_button=dia_filter_list.editmenu(MSG_U(M_PRTFILTERSLIST, "Printer filters' List"), "", help_nil, my_selection, MENUBUT_QUIT);

        steppy=whole_list;
        my_counter=my_selection+1;
        while(my_counter--){
            if(!my_counter)
                strcpy(output_printer_filter, steppy);
            steppy+=strlen(steppy)+1;
        }

        free(whole_list);
    }

    if((my_button==MENU_QUIT)||(my_button==MENU_ESCAPE))
        return(0);
    return(1);
}

/* will reset resolution and colordepth for first permitted values
   (because new filter installed) */
/* given the selected filtername */
/* UNFINISHED */
void new_filter_so_reset_some_things(char *overrider_printer)
{
    char *my_new_gs_driver;
    char *my_new_resolution;
    char *my_new_colordepth;
    char tempy[TYPICAL_STRING_SIZE];
    char tempy2[TYPICAL_STRING_SIZE];

    /* f_printername redefined.. */
    strcpy(f_printername, overrider_printer);

    my_new_gs_driver=return_filter_propertylist(overrider_printer, "GSDriver: ");
    my_new_resolution=return_filter_propertylist(overrider_printer, "Resolution: ");
    my_new_colordepth=return_filter_propertylist(overrider_printer, "BitsPerPixel: ");

    if(!(strcmp(my_new_gs_driver, "ppa"))) /* ppa printers shall not have fast ascii enabled */
        f_fastascii=0;

    /* ps_gsdevice redefined.. */
    strcpy(ps_gsdevice, my_new_gs_driver);

    /* ps_resolution redefined.. */
    gimme_string_element(my_new_resolution, 0, tempy);
    gimme_string_element(my_new_resolution, 1, tempy2);
    sprintf(ps_resolution, "%sx%s", tempy, tempy2);

    /* ps_color redefined.. */
    gimme_string_element(my_new_colordepth, 0, ps_color);

    free(my_new_colordepth);
    free(my_new_resolution);
    free(my_new_gs_driver);
}

/* this was originally mean to be a stand-alone window */
/* if *overrider_printer, remake part of configuration
   (because new printer was selected) */
/* returns pressed button */
void editfilter_window(int which_printer, DIALOG &filterwindow, char *overrider_printer)
{
//    DIALOG filterwindow;
//    MENU_STATUS        my_button;

    FIELD_LIST *pop_resolution;
    FIELD_LIST *pop_papersize;
    FIELD_LIST *pop_colordepth;
    FIELD_LIST *pop_printedpages;

    filterwindow.newf_title(MSG_U(M_FILTOPTS, "Filter options"), 1, "", MSG_R(M_FILTOPTS));
    filterwindow.newf_info(MSG_U(M_FILTCURRSELECTED, "Filter currently selected:"), f_printername);

    /* load gui variables */
    load_to_gui_variables();

    /* if new filter was selected, change some config things.. */
    if(*overrider_printer)
        new_filter_so_reset_some_things(overrider_printer);

    /* if printer could not be located, the loaded values aren't in list.. so select in the list the empty one (otherwise there will be annoying requesters from linuxconf) */
    if(!*f_printername){
        f_resolution.setfrom("");
        f_colordepth.setfrom("");
    }

//    filterwindow.newf_info("Filter description:", f_filterdescription);

    pop_resolution=filterwindow.newf_list(MSG_U(M_FLRESOLUTION, "Resolution:"), f_resolution);
    populate_fieldlist(f_printername, "Resolution: ", pop_resolution, 1);
    pop_papersize=filterwindow.newf_list(MSG_U(M_FLPAPERSIZE, "Paper size:"), f_papersize); {
        pop_papersize->addopt("11x17", MSG_U(M_PAPERTYPE1117, "11x17"), "");
        pop_papersize->addopt("a0", MSG_U(M_PAPERTYPEA0, "A0"), "");
        pop_papersize->addopt("a1", MSG_U(M_PAPERTYPEA1, "A1"), "");
        pop_papersize->addopt("a2", MSG_U(M_PAPERTYPEA2, "A2"), "");
        pop_papersize->addopt("a3", MSG_U(M_PAPERTYPEA3, "A3"), "");
        pop_papersize->addopt("a4", MSG_U(M_PAPERTYPEA4, "A4"), "");
        pop_papersize->addopt("a5", MSG_U(M_PAPERTYPEA5, "A5"), "");
        pop_papersize->addopt("a6", MSG_U(M_PAPERTYPEA6, "A6"), "");
        pop_papersize->addopt("a7", MSG_U(M_PAPERTYPEA7, "A7"), "");
        pop_papersize->addopt("a8", MSG_U(M_PAPERTYPEA8, "A8"), "");
        pop_papersize->addopt("a9", MSG_U(M_PAPERTYPEA9, "A9"), "");
        pop_papersize->addopt("a10", MSG_U(M_PAPERTYPEA10, "A10"), "");
        pop_papersize->addopt("archA", MSG_U(M_PAPERTYPEARA, "archA"), "");
        pop_papersize->addopt("archB", MSG_U(M_PAPERTYPEARB, "archB"), "");
        pop_papersize->addopt("archC", MSG_U(M_PAPERTYPEARC, "archC"), "");
        pop_papersize->addopt("archD", MSG_U(M_PAPERTYPEARD, "archD"), "");
        pop_papersize->addopt("archE", MSG_U(M_PAPERTYPEARE, "archE"), "");
        pop_papersize->addopt("b0", MSG_U(M_PAPERTYPEB0, "B0"), "");
        pop_papersize->addopt("b1", MSG_U(M_PAPERTYPEB1, "B1"), "");
        pop_papersize->addopt("b2", MSG_U(M_PAPERTYPEB2, "B2"), "");
        pop_papersize->addopt("b3", MSG_U(M_PAPERTYPEB3, "B3"), "");
        pop_papersize->addopt("b4", MSG_U(M_PAPERTYPEB4, "B4"), "");
        pop_papersize->addopt("b5", MSG_U(M_PAPERTYPEB5, "B5"), "");
        pop_papersize->addopt("flsa", MSG_U(M_PAPERTYPEFLSA, "flsa"), "");
        pop_papersize->addopt("flse", MSG_U(M_PAPERTYPEFLSE, "flse"), "");
        pop_papersize->addopt("halfletter", MSG_U(M_PAPERTYPEHLET, "halfletter"), "");
        pop_papersize->addopt("ledger", MSG_U(M_PAPERTYPELEDG, "ledger"), "");
        pop_papersize->addopt("legal", MSG_U(M_PAPERTYPELEGAL, "legal"), "");
        pop_papersize->addopt("letter", MSG_U(M_PAPERTYPELETTER, "letter"), "");
        pop_papersize->addopt("note", MSG_U(M_PAPERTYPENOTE, "note"), "");
    }
    pop_colordepth=filterwindow.newf_list(MSG_U(M_FLCOLORDEPTH, "Color depth:"), f_colordepth);
    populate_fieldlist(f_printername, "BitsPerPixel: ", pop_colordepth, 2);
    pop_printedpages=filterwindow.newf_list(MSG_U(M_FLPRINTEDPAGES, "Printed pages:"), f_printedpages);{
        pop_printedpages->addopt("1");
        pop_printedpages->addopt("2");
        pop_printedpages->addopt("4");
        pop_printedpages->addopt("8");
    }

    filterwindow.setbutinfo(MENU_USR1, MSG_U(M_SELECTFILTER, "Select filter..."), MSG_R(M_SELECTFILTER));

    filterwindow.newf_chk("", f_sendeof, MSG_U(M_SENDEOF, "Send EOF"));
    filterwindow.newf_chk("", f_correctcrlf, MSG_U(M_FIXCRLF, "Fix ladder effect (CR+LF)"));
    /* only enable this if printer is *NOT* PPA */
    if(strcmp(ps_gsdevice, "ppa"))
        filterwindow.newf_chk("", f_fastascii, MSG_U(M_FASTASCII, "Fast ASCII output"));

    filterwindow.newf_num(MSG_U(M_HORIZMARGINS, "Horizontal margins:"), f_horizmargins);
    filterwindow.newf_num(MSG_U(M_VERTMARGINS, "Vertical margins:"), f_vertmargins);
    filterwindow.newf_str(MSG_U(M_GSEXTOPTIONS, "GS extra options:"), f_extragsopt, TYPICAL_STRING_SIZE-1);
}



/* edit properties of the given printer */
/* returns true if this printer was changed (or deleted) */
int editprinter_window(int which_printer)
{
    MENU_STATUS    my_button;
    int    my_selection;

    char   maxfilsiz[TYPICAL_STRING_SIZE];
    char   supheaders;
    char   dev_selected;
    char   custom_dev[TYPICAL_STRING_SIZE];

    char   overrider_printer[500]; /* if not empty, means this printer should be used instead */
    char   dotconfig_fullpath[500];

    int    my_printer_type;

    DIALOG dia_editptr;

    /* builds .config full path for this queue */
    sprintf(dotconfig_fullpath, "%s/%s", my_printcap[which_printer].sd, DOTCONFIG_FILENAME);
    *overrider_printer=0;
    my_button=MENU_USR1;


    /* load seconday configfiles */
    load_initialise_secondary_configfiles(which_printer);
    /* load .config, if present */
    load_dotconfig_data(dotconfig_fullpath);

    /* loop if it exits for filter_selection window, because it needs to return control to this window */
    while((my_button==MENU_USR1)||(my_button==MENU_USR2))
    {
    {
//        DIALOG dia_editptr;
#define myprt my_printcap[which_printer]

        dia_editptr.remove_all();

        /* determines printer type */
        my_printer_type=return_printer_type(which_printer);

        dia_editptr.newf_info(MSG_U(M_PRINTERNAME, "Printer name:"), general_printer_index[which_printer].devname);
        {
            char tempz[TYPICAL_STRING_SIZE];

            switch(my_printer_type){
            case PRT_LOCAL:
                strcpy(tempz, MSG_U(M_TYPELOCAL, "local"));
                break;
            case PRT_REMOTE:
                strcpy(tempz, MSG_U(M_TYPEREMOTE, "remote"));
                break;
            case PRT_SMBWIN:
                strcpy(tempz, MSG_U(M_TYPESMBWIN, "SMB/Windows"));
                break;
            case PRT_NETWARE:
                strcpy(tempz, MSG_U(M_TYPENETW, "Netware"));
                break;
            }
            dia_editptr.newf_info(MSG_U(M_PRINTERTYPE, "Printer type:"), tempz);
        }

        dia_editptr.newf_title(MSG_U(M_GENERALOPTS, "General options"), 1, "", MSG_R(M_GENERALOPTS));

        sprintf(maxfilsiz, "%d", myprt.mx);
        dia_editptr.newf_str(MSG_U(M_MAXFILESIZE, "Maximum file size:"), maxfilsiz, 9);
        if(myprt.sh){
            supheaders=1;
        } else {
            supheaders=0;
        }
        dia_editptr.newf_chk(MSG_U(M_SUPRESSHEADERS, "Supress headers:"), supheaders, MSG_U(M_USUALLYYES, "(usually yes)"));

        dia_editptr.newf_title(MSG_U(M_PRINTERTYPESPECIFIC, "Printertype-specific options"), 1, "", MSG_R(M_PRINTERTYPESPECIFIC));

        /* now add the printertype-specific gadgets */
        switch(my_printer_type){
        case PRT_LOCAL:
            dev_selected=3;
            strcpy(custom_dev, myprt.lp);

            if(!strcmp(custom_dev, "/dev/lp0")){
                dev_selected=0;
                *custom_dev=0;
            }
            if(!strcmp(custom_dev, "/dev/lp1")){
                dev_selected=1;
                *custom_dev=0;
            }
            if(!strcmp(custom_dev, "/dev/lp2")){
                dev_selected=2;
                *custom_dev=0;
            }
            dia_editptr.newf_radio(MSG_U(M_LOCALCONNECTEDTO, "Connected to:"), dev_selected, 0, MSG_U(M_LOCALTYPICAL, "/dev/lp0 (typical)"));
            dia_editptr.newf_radio("", dev_selected, 1, "/dev/lp1");
            dia_editptr.newf_radio("", dev_selected, 2, "/dev/lp2");
            dia_editptr.newf_radio("", dev_selected, 3, "custom:");
            dia_editptr.newf_str("", custom_dev, TYPICAL_STRING_SIZE-1);
            break;
        case PRT_REMOTE:
            dia_editptr.newf_str(MSG_U(M_REMOTEMACHINE, "Remote machine:"), myprt.rm, TYPICAL_STRING_SIZE-1);
            dia_editptr.newf_str(MSG_U(M_REMOTEQUEUE, "Remote queue:"), myprt.rp, TYPICAL_STRING_SIZE-1);
            break;
        case PRT_SMBWIN:
            dia_editptr.newf_str(MSG_U(M_SSMBUSER, "User:"), dc_var.user, TYPICAL_STRING_SIZE-1);
            dia_editptr.newf_str(MSG_U(M_SSMBPASSWD, "Password:"), dc_var.password, TYPICAL_STRING_SIZE-1);
            dia_editptr.newf_str(MSG_U(M_SSMBHOSTNAME, "Hostname:"), dc_var.hostname, TYPICAL_STRING_SIZE-1);
            dia_editptr.newf_str(MSG_U(M_SSMBPRTNAME, "Printername:"), dc_var.printername, TYPICAL_STRING_SIZE-1);
            dia_editptr.newf_str(MSG_U(M_SSMBHOSTIP, "Host IP (optional):"), dc_var.hostip, TYPICAL_STRING_SIZE-1);
            dia_editptr.newf_str(MSG_U(M_SSMBWORKGP, "Workgroup:"), dc_var.workgroup, TYPICAL_STRING_SIZE-1);
            break;
        case PRT_NETWARE:
            dia_editptr.newf_str(MSG_U(M_SNCPUSER, "User:"), dc_var.user, TYPICAL_STRING_SIZE-1);
            dia_editptr.newf_str(MSG_U(M_SNCPPASSWD, "Password:"), dc_var.password, TYPICAL_STRING_SIZE-1);
            dia_editptr.newf_str(MSG_U(M_SNCPSERVER, "Server:"), dc_var.server, TYPICAL_STRING_SIZE-1);
            dia_editptr.newf_str(MSG_U(M_SNCPQUEUE, "Queue:"), dc_var.queue, TYPICAL_STRING_SIZE-1);
            break;
        }

        /* adds filter properties */
        /* this was originally meant to have its own window */
        editfilter_window(which_printer, dia_editptr, overrider_printer);

        /* adds 'Print test page' button */
        dia_editptr.setbutinfo(MENU_USR2, MSG_U(X_TEST, "Test"), MSG_R(X_TEST));

        /* --- opens window --- */
        my_selection=0;
        my_button=dia_editptr.edit(MSG_U(M_PRINTERPROP, "Printer properties"), "", my_properties_helpfile, my_selection, MENUBUT_QUIT|MENUBUT_ACCEPT|MENUBUT_DEL|MENUBUT_USR1|MENUBUT_USR2);
    }

    /* if the right button was pressed, opens filter selection window */
    if(my_button==MENU_USR1){
        dia_editptr.save(); /* save contents, otherwise window will be restored to saved values on return (annoying for user) */
        select_printer_filter(overrider_printer);

        /* saves selected filter to file */
        if(*overrider_printer){
            new_filter_so_reset_some_things(overrider_printer);
            load_to_gui_variables();
            /* linux to glupi os, lepiej zrobic na amidze ;) */
//            dia_editptr.reload();
            write_filter_description(which_printer, overrider_printer);
            save_all_the_filters_stuff(which_printer);
            save_dotconfig_data(dotconfig_fullpath);
        }
    }

    /* test filter */
    if(my_button==MENU_USR2){
        /* picks only the first printer name (drop aliases) */
        char devname[TYPICAL_STRING_SIZE];
        char *devname_zeropoint;
        strcpy(devname, general_printer_index[which_printer].devname);
        if((devname_zeropoint=strchr(devname, '|')))
            *devname_zeropoint=0;

        DIALOG my_dialog;
        my_dialog.settype (DIATYPE_POPUP);
        MENU_STATUS my_button2=MENU_NULL;
        int         my_selection2=0;
        SSTRING command_to_send;

        my_dialog.new_menuitem("", MSG_U(X_DOCOMMTEST, "Print ascii test"));
        my_dialog.new_menuitem("", MSG_U(X_DOFILTERTEST, "Print postscript test"));
        my_dialog.new_menuitem("", MSG_U(X_STOPPRINTING, "STOP PRINTING"));

        /* opens dialog.. */
        my_selection2=0;
        my_button2=my_dialog.editmenu(MSG_U(T_PRINTERTEST, "Printer test"), MSG_U(X_PRINTERTESTEXPL, "Select \"Print\" so a testpage will be printed in order\nto check the configuration.\nIf the printer goes out of control,\nselect \"STOP PRINTING\"\n(you may also need to turn off the printer)\n"), help_nil, my_selection2, MENUBUT_QUIT);

        switch(my_button2){
        case MENU_QUIT:
        case MENU_ESCAPE:
            break;
        default:
            switch(my_selection2){
            case 0:
                /* prints test ASCII page */
                /* prints test PS page */
                /* saves the printer data (otherwise it can't be tested) */
                save_all_the_filters_stuff(which_printer);
                save_dotconfig_data(dotconfig_fullpath);
                /* forces lpd restart */
                just_executes_the_proggy("killall lpd");
                just_executes_the_proggy("lpd");
                command_to_send.setfromf("lpc start %s", devname);
                just_executes_the_proggy(command_to_send.get());

                /* sends the test data to printer */
                command_to_send.setfromf(TEST_PRINTER_COMMAND_ASCII, devname);
                just_executes_the_proggy(command_to_send.get());
				xconf_notice (MSG_U(I_DONE,"Done"));
                break;
            case 1:
                /* prints test PS page */
                /* saves the printer data (otherwise it can't be tested) */
                save_all_the_filters_stuff(which_printer);
                save_dotconfig_data(dotconfig_fullpath);
                /* forces lpd restart */
                just_executes_the_proggy("killall lpd");
                just_executes_the_proggy("lpd");
                command_to_send.setfromf("lpc start %s", devname);
                just_executes_the_proggy(command_to_send.get());

                /* sends the test data to printer */
                command_to_send.setfromf(TEST_PRINTER_COMMAND, devname);
                just_executes_the_proggy(command_to_send.get());
				xconf_notice (MSG_R(I_DONE));
                break;
            case 2:
                /* aborts printing, cleans queue and restarts lpd */
                command_to_send.setfromf("lpc abort %s", devname);
                just_executes_the_proggy(command_to_send.get());
                command_to_send.setfromf("lprm -P%s - >/dev/null", devname);
                just_executes_the_proggy(command_to_send.get());
                just_executes_the_proggy("killall lpd");
                just_executes_the_proggy("lpd");
                command_to_send.setfromf("lpc start %s", devname);
                just_executes_the_proggy(command_to_send.get());
				xconf_notice (MSG_R(I_DONE));
                break;
            default:
                break;
            }
            break;
        }
    }

    }

    /* save the filter properties if asked so */
    if(my_button==MENU_ACCEPT){
        save_all_the_filters_stuff(which_printer);
        save_dotconfig_data(dotconfig_fullpath);
    }

    /* converts strings to integer, chars to bools, etc */
    myprt.mx=atoi(maxfilsiz);
    myprt.sh=supheaders;
    /* now converst the printertype-specific things.. */
    switch(my_printer_type){
    case PRT_LOCAL:
        if(dev_selected==0)
            strcpy(myprt.lp, "/dev/lp0");
        if(dev_selected==1)
            strcpy(myprt.lp, "/dev/lp1");
        if(dev_selected==2)
            strcpy(myprt.lp, "/dev/lp2");
        if(dev_selected==3)
            strcpy(myprt.lp, custom_dev);
        break;
    case PRT_REMOTE:
        break;
    case PRT_SMBWIN:
        break;
    case PRT_NETWARE:
        break;
    }

    if(my_button==MENU_DEL){
        if(confirm_yesno_window(MSG_U(M_REMOVPRTTITLE, "Removing printer"), MSG_U(M_REMOVPRTTEXT, "Confirm removing this printer?"))){
            char tempy[500];

            general_printer_index[which_printer].exists=0;
//            (void)save_all_printers_data(); /* let's make sure noone will try to user this printer from now */
            remove_filter_description(which_printer); /* removes filtername from linuxconf global config file */

            /* now removes the printer directory and its files */
            if(myprt.sd){
                if(zaapboom_directory(my_printcap[which_printer].sd)){
                    sprintf(tempy, MSG_U(M_COULDNTREMOVEPRTTX, "Could not remove printer's files and/or its directory.\nYou have to manually remove the following directory:\n%s\n"), my_printcap[which_printer].sd);
                    informational_window(MSG_U(M_COULDNTREMOVEPRTTT, "Unable to remove printer's directory"), tempy);
                }
            }
            return(1);
        }
    }

    if((my_button==MENU_QUIT)||(my_button==MENU_ESCAPE)){
        return(0);
    }
    return(1);
}
