#include <Python.h>
#include "../pythonmod.h"

static PyObject *ErrorObject;

static PyObject *
LCDialog_edit(LCDialogObject *self, PyObject *args)
{
	char *title,*intro;
	int but=MENUBUT_ACCEPT|MENUBUT_CANCEL;
	int ret;
	LCHelpFileObject *helpfile = NULL;
	HELP_FILE *help_file;
	if (!PyArg_ParseTuple(args,"ss|iO",&title,&intro,&but,&helpfile))
		return NULL;
	if (helpfile == NULL)
		help_file = &help_nil;
	else
		help_file = helpfile->helpfile;
	ret = self->dia->edit(title,intro,*help_file,self->nof,but);
	return PyInt_FromLong(ret);
}

static PyObject *
LCDialog_show(LCDialogObject *self, PyObject *args)
{
	char *title,*intro;
	int but=0;
	LCHelpFileObject *helpfile = NULL;
	HELP_FILE *help_file;
	if (!PyArg_ParseTuple(args,"ss|iO",&title,&intro,&but,&helpfile))
		return NULL;
	if (helpfile == NULL)
		help_file = &help_nil;
	else
		help_file = helpfile->helpfile;
	self->dia->show(title,intro,*help_file,self->nof,but);
	diagui_flush();
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject *
LCDialog_reload(LCDialogObject *self, PyObject *args)
{
	if (!PyArg_ParseTuple(args,""))
		return NULL;
	self->dia->reload();
	diagui_flush();
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject *
LCDialog_new_menuitem(LCDialogObject *self, PyObject *args)
{
	char *prompt1,*prompt2,*icon="";
	if (!PyArg_ParseTuple(args,"ss|s",&prompt1,&prompt2,&icon))
		return NULL;
	self->dia->new_menuitem(icon,prompt1,prompt2);
	return PyInt_FromLong((long)prompt2);
}

static PyObject *
LCDialog_new_menuitems(LCDialogObject *self, PyObject *args)
{
	PyObject *l,*ret;
	if (!PyArg_ParseTuple(args,"O",&l))
		return NULL;
	int size = PyList_Size(l);
	ret = PyList_New(size);
	for (int i = 0; i != size; i++) {
		char *prompt1, *prompt2, *icon="";
		PyObject *o = PyList_GetItem(l,i);
		if(!PyArg_ParseTuple(o,"ss|s",&prompt1,&prompt2,&icon))
			return NULL;
		self->dia->new_menuitem(icon,prompt1,prompt2);
		PyList_SetItem(ret,i,PyInt_FromLong((long)prompt2));
	}
	return ret;
}

static PyObject *
LCDialog_newf_str(LCDialogObject *self, PyObject *args)
{
	char *prompt;
	char *str = "";
	if (!PyArg_ParseTuple(args,"s|s",&prompt,&str))
		return NULL;
	SSTRING *sstr = new SSTRING(str);
	self->dia->newf_str(prompt,*sstr);
	PyObject *obj = LCString_FromStringDealloc(sstr);
	PyList_Append(self->reflist,obj);
	return obj;
}

static PyObject *
LCDialog_newf_title(LCDialogObject *self, PyObject *args)
{
	char *prompt, *msg, *pad = "";
	int level = 0;
	if (!PyArg_ParseTuple(args,"ss|is",&prompt,&msg,&level,&pad))
		return NULL;
	self->dia->newf_title(pad,level,prompt,msg);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject *
LCDialog_newf_combo(LCDialogObject *self, PyObject *args)
{
	char *prompt, *str;
	PyObject *l;
	if (!PyArg_ParseTuple(args,"ssO",&prompt,&str,&l))
		return NULL;
	SSTRING *sstr = new SSTRING(str);
	FIELD_COMBO *combo = self->dia->newf_combo(prompt,*sstr);
	int size = PyList_Size(l);
	for (int i = 0; i != size; i++) {
		char *value, *verbose="";
		PyObject *o = PyList_GetItem(l,i);
		if(!PyArg_ParseTuple(o,"s|s",&value,&verbose))
			return NULL;
		combo->addopt(value,verbose);
	}
	PyObject *obj = LCString_FromStringDealloc(sstr);
	PyList_Append(self->reflist,obj);
	return obj;
}

static PyObject *
LCDialog_newf_chk(LCDialogObject *self, PyObject *args)
{
	char *prompt;
	char *c = new char();
	char *title;
	if (!PyArg_ParseTuple(args,"sis",&prompt,c,&title)) {
		delete c;
		return NULL;
	}
	self->dia->newf_chk(prompt,*c,title);
	PyObject *obj = LCChar_FromCharDealloc(c);
	PyList_Append(self->reflist,obj);
	return obj;
}

static PyObject *
LCDialog_newf_chkm(LCDialogObject *self, PyObject *args)
{
	char *prompt;
	char *c = new char();
	PyObject *l;
	if (!PyArg_ParseTuple(args,"siO",&prompt,c,&l)) {
		delete c;
		return NULL;
	}
	int size = PyList_Size(l);
	const char **titles = (const char **) malloc(sizeof(char **)*(size+1));
	int i = 0;
	for (; i != size; i++) {
		PyObject *o = PyList_GetItem(l,i);
		titles[i] = PyString_AsString(o);
	}
	titles[i] = 0;
	self->dia->newf_chkm(prompt,*c,titles);
	PyObject *obj = LCChar_FromCharDealloc(c);
	PyList_Append(self->reflist,obj);
	return obj;
}

static PyObject *
LCDialog_newf_chkm_num(LCDialogObject *self, PyObject *args)
{
	char *prompt;
	int *i = new int();
	PyObject *l;
	if (!PyArg_ParseTuple(args,"siO",&prompt,i,&l)) {
		delete i;
		return NULL;
	}
	int size = PyList_Size(l);
	const char **options = (const char **) malloc(sizeof(char **)*(size+1));
	int *vals = (int *) malloc(sizeof(int *)*(size));
	int n = 0;
	for (; n != size; n++) {
		PyObject *o = PyList_GetItem(l,n);
		if (!PyArg_ParseTuple(o,"is",&vals[n],&options[n])) {
			delete i;
			free(options);
			free(vals);
		}
	}
	options[n] = 0;
	self->dia->newf_chkm_num(prompt,*i,vals,options);
	PyObject *obj = LCInt_FromIntDealloc(i);
	PyList_Append(self->reflist,obj);
	return obj;
}

static PyObject *
LCDialog_newf_chkm_str(LCDialogObject *self, PyObject *args)
{
	char *prompt;
	int *i = new int();
	PyObject *l;
	if (!PyArg_ParseTuple(args,"siO",&prompt,i,&l)) {
		delete i;
		return NULL;
	}
	int size = PyList_Size(l);
	const char **options = (const char **) malloc(sizeof(char **)*(size));
	int *vals = (int *) malloc(sizeof(int *)*(size));
	int n = 0;
	for (; n != size; n++) {
		PyObject *o = PyList_GetItem(l,n);
		if (!PyArg_ParseTuple(o,"is",&vals[n],&options[n])) {
			delete i;
			free(options);
			free(vals);
		}
	}
	n--;
	SSTRING *sstr = new SSTRING(options[n]);
	options[n] = 0;
	self->dia->newf_chkm_str(prompt,*i,*sstr,vals,options);
	PyObject *tup = PyTuple_New(2);
	PyTuple_SetItem(tup,0,LCInt_FromIntDealloc(i));
	PyTuple_SetItem(tup,1,LCString_FromStringDealloc(sstr));
	PyList_Append(self->reflist,tup);
	return tup;
}

static PyObject *
LCDialog_newf_radio(LCDialogObject *self, PyObject *args)
{
	char *prompt, *title;
	int instance;
	LCCharObject *cchar;
	if (!PyArg_ParseTuple(args,"ssOi",&prompt,&title,&cchar,&instance))
		return NULL;
	self->dia->newf_radio(prompt,*cchar->c,instance,title);
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject *
LCDialog_newf_gauge(LCDialogObject *self, PyObject *args)
{
	char *prompt;
	int *val = new int();
	int range;
	if (!PyArg_ParseTuple(args,"sii",&prompt,val,&range)) {
		delete val;
		return NULL;
	}
	self->dia->newf_gauge(prompt,*val,range);
	PyObject *obj = LCInt_FromIntDealloc(val);
	PyList_Append(self->reflist, obj);
	return obj;
}

static struct PyMethodDef LCDialog_methods[] = {
	{"edit", (binaryfunc) LCDialog_edit, 1},
	{"show", (binaryfunc) LCDialog_show, 1},
	{"reload", (binaryfunc) LCDialog_reload, 1},
	{"new_menuitem", (binaryfunc) LCDialog_new_menuitem, 1},
	{"new_menuitems", (binaryfunc) LCDialog_new_menuitems, 1},
	{"newf_str", (binaryfunc) LCDialog_newf_str, 1},
	{"newf_title", (binaryfunc) LCDialog_newf_title, 1},
	{"newf_combo", (binaryfunc) LCDialog_newf_combo, 1},
	{"newf_chk", (binaryfunc) LCDialog_newf_chk, 1},
	{"newf_chkm", (binaryfunc) LCDialog_newf_chkm, 1},
	{"newf_chkm_num", (binaryfunc) LCDialog_newf_chkm_num, 1},
	{"newf_chkm_str", (binaryfunc) LCDialog_newf_chkm_str, 1},
	{"newf_radio", (binaryfunc) LCDialog_newf_radio, 1},
	{"newf_gauge", (binaryfunc) LCDialog_newf_gauge, 1},
	{NULL, NULL}
};

static void
LCDialog_dealloc(LCDialogObject *self)
{
	if (self->owner)
		delete self->dia;
	Py_DECREF(self->reflist);
	PyMem_DEL(self);
}

static PyObject *
LCDialog_getattr(LCDialogObject *self, char *name)
{
	if (strcmp(name,"nof") == 0)
		return PyInt_FromLong(self->nof);
	return Py_FindMethod(LCDialog_methods, (PyObject*) self, name);
}

static int
LCDialog_setattr(LCDialogObject *self, char *name, PyObject *obj)
{
	if (obj != NULL && strcmp(name,"nof") == 0)
		self->nof = PyInt_AsLong(obj);
		return 0;
	return -1;
}

PyTypeObject LCDialog_Type = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,
	"LCDialog",
	sizeof(LCDialogObject),
	0,
	(destructor) LCDialog_dealloc,
	(printfunc) 0,
	(getattrfunc) LCDialog_getattr,
	(setattrfunc) LCDialog_setattr,
	(cmpfunc) 0,
	(reprfunc) 0,
	0,
	0,
	0,
	(hashfunc) 0,
	(ternaryfunc) 0,
	(reprfunc) 0
};

PyObject *
LCDialog_New()
{
	LCDialogObject *self;
	self = PyObject_NEW(LCDialogObject, &LCDialog_Type);
	if (self == NULL)
		return NULL;
	self->dia = new DIALOG();
	self->owner = 1;
	self->nof = 0;
	self->reflist = PyList_New(0);
	return (PyObject*) self;
}

PyObject *
LCDialog_FromDialog(DIALOG *dia)
{
	LCDialogObject *self;
	self = PyObject_NEW(LCDialogObject, &LCDialog_Type);
	if (self == NULL)
		return NULL;
	self->dia = dia;
	self->owner = 0;
	self->nof = 0;
	self->reflist = PyList_New(0);
	return (PyObject*) self;
}

PyObject *
LCDialog_FromDialogDealloc(DIALOG *dia)
{
	LCDialogObject *self;
	self = PyObject_NEW(LCDialogObject, &LCDialog_Type);
	if (self == NULL)
		return NULL;
	self->dia = dia;
	self->owner = 0;
	self->nof = 0;
	self->reflist = PyList_New(0);
	return (PyObject*) self;
}

static PyObject *
lcdialog_LCDialog(PyObject *self, PyObject *args)
{
	if (!PyArg_ParseTuple(args, ""))
		return NULL;
	return LCDialog_New();
}

static PyObject *
lcdialog_dialog_mode(PyObject *self, PyObject *args)
{
	if (!PyArg_ParseTuple(args, ""))
		return NULL;
	return PyInt_FromLong(dialog_mode);
}

static struct PyMethodDef lcdialog_methods[] = {
	{"LCDialog", lcdialog_LCDialog, 1},
	{"dialog_mode", lcdialog_dialog_mode, 1},
	{NULL,NULL}
};

extern "C" void
initlcdialog()
{
	PyObject *m, *d;
	m = Py_InitModule("lcdialog", lcdialog_methods);
	d = PyModule_GetDict(m);
	ErrorObject = Py_BuildValue("s", "lcdialog.error");
	PyDict_SetItemString(d, "error", ErrorObject);
	if (PyErr_Occurred())
		Py_FatalError("can't initialize module lcdialog");
}
