#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <translat.h>
#include "shellmod.m"
#include "shellmod.h"
#include "protocol.h"
#include "../../paths.h"

static void usage()
{
	fprintf (stderr
		,MSG_U(I_PRGUSAGE
			,"shellmod [ --debug ] script [ options ]\n"
			 "\n"
			 "\tRun a Linuxconf module/script.\n"
			 "shellmod --build\n"
			 "\n"
			 "\tEnter the builder\n")
		); 
}

static int shellprg_runmain (
	const char *cmd,
	char *argv[],
	int start,
	int end)
{
	int ret = -1;
	SHELL_INSTANCE inst (cmd);
	if (inst.isalive()){
		SSTRING buf ("main ");
		for (int i=start; i<end; i++){
			buf.append ("\"");
			buf.append (argv[i]);
			buf.append ("\" ");
		}
		char cdcmd[PATH_MAX],cwd[PATH_MAX];
		snprintf (cdcmd,PATH_MAX-1,"cd %s",getcwd(cwd,sizeof(cwd)-1));
		inst.runfunc (cdcmd);
		ret = inst.runfunc (buf.get());
	}
	return ret;
}

static int shellprg_setabs (const char *relpath, char abspath[PATH_MAX])
{
	int ret = 0;
	if (relpath[0] != '/'){
		char cwdpath[PATH_MAX];
		const char *cwd = getcwd(cwdpath,sizeof(cwdpath));
		if (cwd == NULL){
			fprintf (stderr,MSG_U(E_CWD,"Can't find current working directory, odd!\n"));
			ret = -1;
		}else{
			snprintf (abspath,PATH_MAX-1,"%s/%s",cwdpath,relpath);
		}
	}else{
		strcpy (abspath,relpath);
	}
	return ret;
}

static int shellprg_exec (
	const char *command,
	int argc,
	char *argv[])
{
	int ret = -1;
	char abspath[PATH_MAX];
	if (shellprg_setabs (argv[0],abspath)!=-1){
		char cmd[PATH_MAX];
		snprintf (cmd,sizeof(cmd)-1,"%s  %s",command,abspath);
		ret = shellprg_runmain (cmd,argv,1,argc);
	}
	return ret;
}


int main (int argc, char *argv[])
{
	int ret = -1;
	linuxconf_loadlibmsg("/usr/lib/linuxconf");
	linuxconf_loadmsg ("shellmod",PACKAGE_REV);
	argc = dialog_parseuioptions(argc,argv);
	translat_checkmissing();

	const char *pt = strrchr(argv[0],'/');
	if (pt != NULL){
		pt++;
	}else{
		pt = argv[0];
	}
	#if 0
		for (int i=0; i<argc; i++){
			printf ("argv[%d] = :%s:\n",i,argv[i]);
		}
	#endif
	if (argc < 2){
		usage();
	}else if (strcmp(pt,"shellmod")==0){
		if (argc == 2 && strcmp(argv[1],"--build")==0){
			shellmod_build();
		}else{
			const char *cmd = "/bin/sh";
			int start = 1;
			while (argv[start] != NULL){
				if (strcmp(argv[start],"--debug")==0){
					protocol_setdebug (true);
					start++;
				}else if (strcmp(argv[start],"--perl")==0){
					cmd = "/usr/bin/perl";
					start++;
				}else{
					break;
				}
			}
			shellprg_exec (cmd,argc-start,argv+start);
		}
	}else{
		shellprg_exec ("/bin/sh",argc-1,argv+1);
	}
	return ret;
}

