
/*
 routines used in several parts of wineconf module
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
// #include "wineconf.h"
// #include "wineconf.m"
#include <fviews.h>

#include "wineconf_global_defs.h"

/* creates a 'boolean' gadget for Y, N or \0 */
void create_triple_boolean_gadget(DIALOG &given_dialog, FIELD_LIST **given_field_list, const char *given_fieldname, SSTRING &given_sstring)
{
    *given_field_list=given_dialog.newf_list(given_fieldname, given_sstring);
    (*given_field_list)->addopt("", MSG_U(X_USEDEFAULT, "(use default)"), "");
    (*given_field_list)->addopt("Y", MSG_U(X_YES, "Yes"), "");
    (*given_field_list)->addopt("N", MSG_U(X_NO, "No"), "");
}


/* processed the given data returning Y, N or '\0' */
void load_boolean_sstring(const char *where_to_read_from, SSTRING &where_to_write)
{
    char tempbuf[2];
    char given_char;

    tempbuf[1]=0;
    if(where_to_read_from){
        given_char=*where_to_read_from;
        switch(given_char){
        case 'Y':
        case 'y':
        case 'T':
        case 't':
        case '1':
            tempbuf[0]='Y';
            break;
        case 'N':
        case 'n':
        case 'F':
        case 'f':
        case '0':
            tempbuf[0]='N';
            break;
        default:
            tempbuf[0]=0;
        }
    } else {
        tempbuf[0]=0;
    }

    where_to_write.setfrom(tempbuf);
}

/* convert given string to higher case, return the same given pointer */
/* applies only to ascii a-z */
char *convert_string_to_highercase(char *given_string)
{
    char *my_counter;
    char my_data;

    my_counter=given_string;
    while((my_data=*my_counter)){
        if((my_data>96)&&(my_data<123))
            *my_counter=my_data-32;
        my_counter++;
    }
    return(given_string);
}

/* convert given string to lowercase, return the same given pointer */
/* applies only to ascii A-Z */
char *convert_string_to_lowercase(char *given_string)
{
    char *my_counter;
    char my_data;

    my_counter=given_string;
    while((my_data=*my_counter)){
        if((my_data>64)&&(my_data<91))
            *my_counter=my_data+32;
        my_counter++;
    }
    return(given_string);
}

/* returns line number of first occurence of the given prefix in a line,
 given the delimiters, including the lines they point to.
 (they also can be defined to the first and last
 lines so it'll seek the whole file) */
/* will return -1 if not found (0 is the first line) */
/* if locate_var==TRUE makes sure the provided prefix is the whole var name,
   otherwise if just a normal prefix search */
int return_line_of_first_occurence(const char *seeken_prefix, VIEWITEMS &given_vitems, const int from_this_line, const int to_this_line, const int locate_var)
{
    VIEWITEM *specific_item;
    int where_i_am;
    int size_of_given_prefix;
    int value_to_return=-1;


    where_i_am=from_this_line;
    size_of_given_prefix=strlen(seeken_prefix);

    while(where_i_am<(to_this_line+1)){
        char seeken_prefix2[size_of_given_prefix+1];
        char readen_prefix[size_of_given_prefix+2]; // + next char + \0
        int  size_for_copied_line; // the readen line is not fully copied

        specific_item=given_vitems.getitem(where_i_am);

        /* converts both to lower case */
        size_for_copied_line=strlen(specific_item->line.get());
        if(size_for_copied_line>size_of_given_prefix)
            size_for_copied_line=size_of_given_prefix+1;
        strcpy(seeken_prefix2, seeken_prefix);
        strncpy(readen_prefix, specific_item->line.get(), size_for_copied_line);
        convert_string_to_lowercase(seeken_prefix2);
        convert_string_to_lowercase(readen_prefix);

//        if(!(strncmp(specific_item->line.get(), seeken_prefix, size_of_given_prefix))){
        if(!(strncmp(readen_prefix, seeken_prefix2, size_of_given_prefix))){
            if(!locate_var){
                // normal prefix search
                value_to_return=where_i_am;
                where_i_am=to_this_line; /* so it'll exit the loop */
            } else {
                // search a variable, let's make some more tests..
                if(size_for_copied_line>=size_of_given_prefix){
                    if(size_for_copied_line==size_of_given_prefix){
                        value_to_return=where_i_am;
                        where_i_am=to_this_line; /* so it'll exit the loop */
                    } else {
                        char pseudo_last_char;

                        pseudo_last_char=*(readen_prefix+size_of_given_prefix);
                        if((pseudo_last_char==' ')||(pseudo_last_char=='=')){
                            value_to_return=where_i_am;
                            where_i_am=to_this_line; /* so it'll exit the loop */
                        }
                    }
                }
            }
        }
        where_i_am++;
    }

    return(value_to_return);
}


/* fill those two variables with the lines where the group is contained
 (both lines pointed are also part of this group, its_first_line returns
 the group identificator tag) */
/* returns TRUE if ok, FALSE if group not found */
int locate_group_limits(const char *groupname, int *its_first_line, int *its_last_line, VIEWITEMS &given_vitems)
{
    int total_items;
    char *my_buff; // groupname plus brackets []
    int group_begins_at=0;
    int group_ends_at=0;

    if((my_buff=(char *)malloc(strlen(groupname)+3))){
        sprintf(my_buff, "[%s]", groupname);
        total_items=given_vitems.getnb();

        group_begins_at=return_line_of_first_occurence(my_buff, given_vitems, 0, total_items-1, 0);
        if(group_begins_at==-1) // oh, group not found
            return(0);

        if(group_begins_at==(total_items-1)){
            group_ends_at=group_begins_at;
        } else {
            group_ends_at=(return_line_of_first_occurence("[", given_vitems, group_begins_at+1, total_items-1, 0)-1);
            if(group_ends_at==-2)
                group_ends_at=total_items-1;
        }

        free(my_buff);
    }

    *its_first_line=group_begins_at;
    *its_last_line=group_ends_at;
    return(1); // ok, things were fine
}

/* return data from a variable from a group */
/* returns null if variable/group not found */
/* NOTE: look at /etc/wine.conf structure to understand wine's concept of
   group on its config file */
const char *return_var_data(const char *groupname, const char *varname, VIEWITEMS &given_vitems)
{
    int group_begins_here;
    int group_ends_here;
    int where_is_that_var;
    VIEWITEM *my_viewitem;
    const char *return_this_data;

    if(!locate_group_limits(groupname, &group_begins_here, &group_ends_here, given_vitems))
        return(NULL);  // group not present
    where_is_that_var=return_line_of_first_occurence(varname, given_vitems, group_begins_here, group_ends_here, 1);
    if(where_is_that_var==-1)
        return(NULL);  // group present but the variable isn't
    my_viewitem=given_vitems.getitem(where_is_that_var);
    return_this_data=my_viewitem->line.get();

    /* get the first data character (after varname and '=') */
    if(!(return_this_data=strchr(return_this_data, '=')))
    {
        // '=' not found, returns empty string
        return_this_data+=strlen(return_this_data);
        return(return_this_data);
    }
    return_this_data++;
    while((*return_this_data)&&(*return_this_data<33)){
        return_this_data++;
    }
    return(return_this_data);
}

void insert_a_line_at(int where_to_insert, const char *given_data, VIEWITEMS &given_vitems)
{
    VIEWITEM *temporary_line=new VIEWITEM(given_data,VIEWITEM_VARIABLE);

    temporary_line->line.setfrom(given_data);
    //temporary_line->type = VIEWITEM_VARIABLE;
    if(where_to_insert>=given_vitems.getnb()){
        given_vitems.add(temporary_line);
    } else {
        given_vitems.insert(where_to_insert, temporary_line);
    }
    //xconf_notice("%x",given_vitems.getitem(0));
//    given_vitems.insert(where_to_insert,temporary_line);
}

/* update/create variable given group and varname (creates group and/or variable if needed) */
/* if remove_if_empty==TRUE removes the variable if its value is "" */
void write_var_data(const char *groupname, const char *varname, const char *given_data, VIEWITEMS &given_vitems, int remove_if_empty)
{
    int where_is_that_var;
    int group_begins_here;
    int group_ends_here;
    int total_items;

    total_items=given_vitems.getnb();

    if(!locate_group_limits(groupname, &group_begins_here, &group_ends_here, given_vitems)){
        // group not found
        {
            char *corrected_groupname;
            if((corrected_groupname=(char *)malloc(strlen(groupname)+3))){
                strcpy(corrected_groupname, "[");
                strcat(corrected_groupname, groupname);
                strcat(corrected_groupname, "]");
                insert_a_line_at(total_items, corrected_groupname, given_vitems);
                free(corrected_groupname);
            }
        }
//        insert_a_line_at(total_items, groupname, given_vitems);
        locate_group_limits(groupname, &group_begins_here, &group_ends_here, given_vitems);
    }

    where_is_that_var=return_line_of_first_occurence(varname, given_vitems, group_begins_here, group_ends_here, 1);
    if(where_is_that_var==-1){
        // var not found
        insert_a_line_at(group_begins_here+1, varname, given_vitems);
        where_is_that_var=group_begins_here+1;
    }

    {
        VIEWITEM *my_viewitem;


        if(remove_if_empty&&(!*given_data)){
            given_vitems.remove_del(where_is_that_var);
        } else {
            my_viewitem=given_vitems.getitem(where_is_that_var);
            my_viewitem->line.setfrom(varname);
            my_viewitem->line.append("=");
            my_viewitem->line.append(given_data);
        }
    }
}

void strgname(SSTRING &givenstr)
{
    if(!(strcmp(givenstr.get(), "wineconfcredits"))){
        int nothing=0;
        DIALOG uhoh_dialog;
        uhoh_dialog.editmenu("lame easter egg", "this module was created by\nDaniel Mealha Cabrita\n(dancab@conectiva.com)\nAugust, 2000", help_nil, nothing, 0);
    }
}

/* puts the first value from the provided 'array' to variable
 if the value is present in this array (see win95/vfat case
 of /etc/wine.conf file */
/* example: "fat\0msdos\0dos\0\0vfat\0win95\0\0\0" */
void adjust_multialiased_string(SSTRING &string_to_adjust, char * array_of_aliases)
{
    const char *provided_string;
    const char *array_reader;
    const char *first_of_group;

    provided_string=string_to_adjust.get();
    array_reader=array_of_aliases;

    while(*array_reader){
        first_of_group=array_reader;
        while(*array_reader){
//            printf("array_reader: [%s]\n", array_reader);
            if(!(strcmp(array_reader, provided_string)))
                string_to_adjust.setfrom(first_of_group);
            array_reader=strchr(array_reader, 0)+1;
//            array_reader+=strlen(array_reader)+1;
        }
        array_reader++;
    }
}

/* remove an entire group from file (used in drives management section) */
void remove_group(char *given_groupname, VIEWITEMS &given_vitems)
{
    int its_first_line=0;
    int its_last_line=0;
    int my_counter=0;

    if(locate_group_limits(given_groupname, &its_first_line, &its_last_line, given_vitems)){
        my_counter=its_last_line;

        while(my_counter>=its_first_line){
            given_vitems.remove_del(my_counter--);
        }
    }
}


